 /**********************************************************************
 *
 * langaction.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * A component of the Greenstone digital library software
 * from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *********************************************************************/

#include <stdio.h>
#include <fstream.h>
#include "langaction.h"
#include "fileutil.h"
#include "gsdltools.h"
#include "gsdlunicode.h"
#include "langdb.h"
#include "receptionist.h"
#include "errno.h"
#include <sys/utsname.h>
 
/*====================*
 * GLOBAL DEFINITIONS *
 *====================*/

// text file reading buffer
const int BUFFSIZE = 1024;
char temp[BUFFSIZE];



/*========================*
 * LOCAL UTILITY ROUTINES *
 *========================*/

// gets the list of packages/files that contain the content of the web-forms
text_tarray get_pagenos(text_t dir, ostream &logout)
{
  text_tarray pagenos;

  // file 'logfile' contains all the page numbers, one number per line
  text_t logfile = filename_cat(dir, "package_forms", "pageno.log");
  
  //open the file as a read only file
  ifstream readfile(logfile.getcstr(), ios::in);
  
  // if file isn't opened, print error and return
  if (!readfile) {
    logout << "File " << logfile.getcstr() << " could not be found.\n";
    return pagenos;
  }
  
  // go through file, pushing each page number on to the end of a text_tarray
  while (readfile.getline(temp, BUFFSIZE-1)) {
    text_t t_temp = temp;
    pagenos.push_back(temp);
  }
  readfile.close();
    
  //returns the array of page numbers
  return pagenos;
}

bool text_t_substring (text_t &text, text_t &substr)
{
  int SIZE = substr.size();

  if (text.size() >= SIZE) {
    if (substr <= text) {
      if (text[0] == substr[0]) {
	text_t temp = "";
	// copy first SIZE elements of 'text' into 'temp'
	for (int size = 0; size < SIZE; size++) {
	  temp.push_back(text[size]);
	}
	if (temp == substr)
	  return true;
      }
    }
  }

  return false;
}


// set_navbarmacros sets _navigationbar_ and _httpbrowseXXX_ macros
// reponse contains 1 metadata field (Title)

bool langaction::init (ostream & /*logout*/) {

  // set up GSDLOS, GSDLHOME and PATH environment variables
  text_t gsdlos, path;
  unsigned int path_separator = ':';
#if defined (__WIN32__)
  gsdlos = "windows";
  path_separator = ';';

  path = filename_cat (gsdlhome, "bin", "windows", "perl", "bin;");

#else
  struct utsname *buf = new struct utsname();
  int i = uname (buf);
  if (i == -1) gsdlos = "linux"; // uname failed
  else gsdlos.setcstr (buf->sysname);
  delete buf;
  lc (gsdlos);
#endif

  pathc = getenv ("PATH");
  path += filename_cat (gsdlhome, "bin", gsdlos);
  path.push_back (path_separator);
  path += filename_cat (gsdlhome, "bin", "script");
  if (pathc != NULL) {
    path.push_back (path_separator);
    path += pathc;
  }
  path = "PATH=" + path;

  gsdlos = "GSDLOS=" + gsdlos;
  text_t setgsdlhome = "GSDLHOME=" + gsdlhome;

  // these will be cleaned up in the destructor
  gsdlosc = gsdlos.getcstr();
  gsdlhomec = setgsdlhome.getcstr();
  pathc = path.getcstr();
  
  putenv (gsdlosc);
  putenv (gsdlhomec);
  putenv (pathc);

  return true;
}
/*====================*
 * CLASS DEFINITIONS  *
 *====================*/

langaction::langaction () {

  package = "";

  cgiarginfo arg_ainfo;

  arg_ainfo.shortname = "l";
  arg_ainfo.longname = "interface language";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "en";
  arg_ainfo.savedarginfo = cgiarginfo::must;
  argsinfo.addarginfo (&cerr, arg_ainfo);

  arg_ainfo.shortname = "bl";
  arg_ainfo.longname = "base language";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "english";
  arg_ainfo.savedarginfo = cgiarginfo::must;
  argsinfo.addarginfo (&cerr, arg_ainfo);

  arg_ainfo.shortname = "tlng";
  arg_ainfo.longname = "translation language";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "french";
  arg_ainfo.savedarginfo = cgiarginfo::must;
  argsinfo.addarginfo (&cerr, arg_ainfo);

}

langaction::~langaction () {
}

bool langaction::check_cgiargs (cgiargsinfoclass &/*argsinfo*/, cgiargsclass &args, 
				ostream &logout) {
  // authenticate the user before allowing modifications
  //  args["uan"] = 1;
  //  args["ug"] = "langadmin";

  return true;
}

void langaction::get_cgihead_info (cgiargsclass &/*args*/, recptprotolistclass * /*protos*/,
				   response_t &response,text_t &response_data, 
				   ostream & logout) {
  response = content;
  response_data = "text/html";
}

void langaction::define_internal_macros (displayclass &disp, cgiargsclass &args, 
					 recptprotolistclass *protos, ostream &logout) {
  
  text_t text = "";
  text_t cmd = "";
  text_t &arg_p = args["p"];
  text_tarray pagenos;
  text_t dir = filename_cat(gsdlhome, "tmp", "lang");
  text_t pageno ="";
  text_t lang = "";
  text_t baselanguage = "";
  
  //creates output stream to argsfile (dir is gsdlhome/tmp/lang)
  text_t argsfile = filename_cat(dir, "arguments.arg");
  ofstream argsout(argsfile.getcstr(), ios::out);
  
  //informs user and returns if output stream not opened
  if(!argsout) {
    logout << "File " << argsfile.getcstr() << " could not be opened\n";
      return;
  }
  
  //ensures access permissions make the file world writable
  text_t changemode = "chmod a+w ";
  changemode += argsfile;
  gsdl_system(changemode, false, logout);
  
  //write out the cgiargsclass passes as a parameter to the subroutine to the argsout file
  argsout << args;
  //close the argsout output stream
  argsout.close();
 
  //sets the current foreign language
  if (args["ownchoice"] != "") {
    lang = args["ownchoice"];
  }
  //  else if (args["hiddenlanguage"] != "") {
//      lang = args["hiddenlanguage"];
//    }
//    else {
//      lang = args["language"];
//    }
  else if (args["tlng"] != "") {
    lang = args["tlng"];
  }

  if (args["bl"] != "") {
    baselanguage = args["bl"];
  }

  //sets current base language
  //  if (args["baselanguage"] != "") {
  //  baselanguage = args["baselanguage"];
  //}
  
  // argument for page is of the form 'macrofile_pageno' so
  // we need to split the argument to get the proper page content
  text_tarray splitarray;
  splitchar(arg_p.begin(), arg_p.end(), '_', splitarray);
  
  arg_p = splitarray[0];
  if (splitarray.size() > 1)
    pageno = splitarray[splitarray.size()-1];

  //if are entering page from macro search
  if (args["macroname"] != "") {
    text_t macroname = args["macroname"];
    
    //calls the translator file to create HTML files
    cmd = "perl " + filename_cat(gsdlhome, "bin", "script", "fromsearch.pl") + " " + macroname + " " + lang + " " + baselanguage;
    logout << "COMMAND: " << cmd.getcstr() <<endl;
    
    if ((gsdl_system(cmd, true, logout)) != 0) {
      logout << "Process " << cmd.getcstr() << " did not execute.../;-D\n";
      return;
    }
    
    define_webpage(disp, protos, logout, pageno, lang, dir, pagenos, text, baselanguage);
    return;
  }
  if (pageno == "fromsearch")
    pageno = "thankyou";

  if (pageno == "picklanguage") {
    pageno = "1";
    
    //THIS IS ONLY COMMENTED COS CVS IS SCREWED! NEED TO UNCOMMENT @ SOME POINT!!!
    //calls the translator file to create HTML files
    
    cmd = "perl " + filename_cat(gsdlhome, "bin", "script", "translator.pl") + " " + lang + " " + baselanguage;
    if ((gsdl_system(cmd, true, logout)) != 0) {
      logout << "Process " << cmd.getcstr() << " did not execute.../;-D\n";
      return;
    }
  }

  //generates the picklanguage page
  if (pageno.empty()) {
    pageno = "picklanguage";
    logout << "@ the 1st pg\n";
    cmd = "perl " + filename_cat(gsdlhome, "bin", "script", "picklanguage.pl");
    logout << cmd.getcstr() << " PICKLANGUAGE\n" <<endl;
    if(gsdl_system(cmd,true,logout) != 0)
      return;
    define_webpage(disp, protos, logout, pageno, lang, dir, pagenos, text, baselanguage);
    
    return;
  }

  //if no more pages will set pageno to thankyou script
  pagenos = get_pagenos(dir,logout);
  
  if (pagenos.empty() && pageno == "")
    pageno = "thankyou";
 
  text_t &submitargs = args[pageno];
  text_t submit = "SUBMIT TRANSLATION";
  
  //if the SUBMIT TRANSLATION >> button has been pushed
  if (text_t_substring(submitargs,submit)) {
    
    //create the command line for submitting the translation
    cmd = "perl " + filename_cat(gsdlhome, "bin", "script", "submit_translation.pl") + " ";
    cmd += (argsfile + " " + lang + " " + baselanguage);
    
    //give command to system to execute submission of translation
    if((gsdl_system(cmd, true, logout)) != 0)
      logout << "Process " << cmd.getcstr() << " did not execute\n";
    
    //increments which page is displayed when translation is submitted
    for (text_tarray::iterator page_itr = pagenos.begin(); 
	 page_itr != pagenos.end(); 
	 page_itr++) {
      if (*page_itr == pageno) {
	page_itr++;
	if (page_itr != pagenos.end())
	  pageno = *page_itr;
	else
	  pageno = "thankyou";
	
	break;	
      }
    }
  }

  pagenos = get_pagenos(dir,logout);
  define_webpage(disp, protos, logout, pageno, lang, dir, pagenos, text, baselanguage);
}

void langaction::define_webpage (displayclass &disp, 
				 recptprotolistclass *protos, ostream &logout,
				 text_t pageno, text_t &lang, text_t &dir, 
				 text_tarray &pagenos, text_t &text, text_t &baselanguage) {
  
  // produces HTML page with the current pages contents
  // formfile is the file that contains web-form for the
  // current package whose name is stored in 'package'

  text_t formfile = filename_cat(dir, "package_forms");
  formfile += ("/" + pageno + ".lang");
    
  ifstream readfile(formfile.getcstr(), ios::in);
  if (!readfile) {
    logout << "File " << formfile.getcstr() << " could not be found.\n";
    return;
  }
  
  
  /* ============================ START WEB-FORM HTML CODE ============================ */
  text += "<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"></head><body>";

  text += "\n<input type=hidden name=\"a\" value=\"lang\">";
  text += "\n<input type=hidden name=\"p\" value=\"translang_";
  text += pageno;
  text += "\">\n";
  text += "\n<input type=hidden name=\"e\" value=\"_decodedcompressedoptions_\">";
  
  text += "_texttranshead_";       

  //if not a special case page display translation info
  if ((pageno != "thankyou") && (pageno !="picklanguage") && (pageno != "fromsearch")) {
  
    text += "<center><strong> _textcurrpage_ ";
    text += pageno;
    text += " _textof_ ";
    text += pagenos.size();
    text += "</strong></center><p><p>\n"; 
  }
  
  //read in HTML from file
  while(readfile.getline(temp, BUFFSIZE-1)) {
    text_t t_temp = temp;
    text += t_temp;
    text += "\n";
  }
  readfile.close();
  
  //if not a special case page display translation info
  if ((pageno != "thankyou") && (pageno != "picklanguage") && (pageno != "fromsearch")) {
    text += "<center><strong> _textcurrpage_ ";
    text += pageno;
    text += " _textof_ ";
    text += pagenos.size();
    text += "</strong><br>\n";

   text_t temp2 = "";

   for (text_tarray::iterator page_itr = pagenos.begin(); 
	page_itr != pagenos.end(); 
	page_itr++) {
    
     if (*page_itr == pageno) {
       page_itr++;
       if (page_itr != pagenos.end())
	 temp2 = *page_itr;
       else
	 temp2 = "thankyou";
       
       text += "<a href=\"_gwcgi_?a=lang&p=translang_";
       text += temp2;
       text += "&baselanguage=";
       text += baselanguage;
       text += "&language=";
       text += lang;
       text += "\"><strong>";
       text += "NEXT>>";
       text += "</strong></a>\n";

       break;	
     }
     else {
       text += "<a href=\"_gwcgi_?a=lang&p=translang_";
       text += *page_itr;
       text += "&baselanguage=";
       text += baselanguage;
       text += "&language=";
       text += lang;
       text += "\"><strong>";
       text += *page_itr;
       text += "</strong></a>\n";
     }
   }
  }

  if (pageno == "fromsearch")
    text += "<center><strong> _textcurrpage_ 1 _textof_ 1</strong><br>\n";

  text += "</body></html>\n";
  
  
  /* ============================ END WEB-FORM HTML CODE ============================ */
  
  
  // THE FOLLOWING MACROS ARE DEFINED IN translang.dm MACRO FILE
  // set the formcontent  as the content of the form-page
 
  logout << text.getcstr() <<endl;

  disp.setmacro("formcontent", "translang", text);
  
  // set the action for the form
  //  disp.setmacro("formaction", "translang", "/cgi-bin/dg5/library");
  disp.setmacro("formaction", "translang", "_gwcgi_");

  text_t navigationbar = "<!-- Navigation Bar -->\n";
  navigationbar += "<table width=\"100%\" cellspacing=0 cellpadding=0>";
  navigationbar += "<tr valign=top><td rowspan=2 align=left>_imagecollection_</td>";
  navigationbar += "<td align=right>_javalinks_</td><td>";

  //PUT THIS LINE BACK IN TO RE CREATE THE LINK TO THE MACRO SEARCH FACILITY
  //  if (pageno != "picklanguage")
  //  navigationbar += "_imageserch_";
  navigationbar += "</td></tr></table>\n</nobr>\n";
  navigationbar += "<!-- End of Navigation Bar -->\n";
  disp.setmacro ("navigationbar", "translang", navigationbar);

}

bool langaction::do_action (cgiargsclass &args, recptprotolistclass * /*protos*/, 
			    browsermapclass * /*browsers*/, displayclass &disp, 
			    outconvertclass &outconvert, ostream &textout, 
			    ostream &logout) {
  text_t &arg_p = args["p"];
  
  outconvertclass text2utf8;

  textout << text2utf8 << disp << ("_" + arg_p + ":header_\n")
	  << ("_" + arg_p + ":content_\n")
	  << ("_" + arg_p + ":footer_\n");

  return true;
}


